/*
 *  Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _REGION_WRAP_P_H_
#define _REGION_WRAP_P_H_

#include <list>

#include "GTLCore/Region.h"
#include "GTLCore/wrappers/Allocate.h"
#include "GTLCore/wrappers/ArrayWrap.h"
#include "OpenShiva/Export.h"

//---------------------- WARNING ----------------------//
// Whenever the following structure is edited,         //
// it's llvm declaration must be changed too in        //
// Wrapper::createRegionType !                         //
//---------------------- WARNING ----------------------//
/**
 * @internal
 * @ingroup OpenShiva
 * Wrapping structure to manipulate a region
 */
struct RegionWrap {
  STRUCT_HEADER
  float x, y, columns, rows;
};

extern "C" {
  /**
   * @internal
   * @ingroup OpenShiva
   * Wrapper to return the left of a region
   */
  OPENSHIVA_EXPORT float region_wrap_left( RegionWrap* self );
  /**
   * @internal
   * @ingroup OpenShiva
   * Wrapper to return the top of a region
   */
  OPENSHIVA_EXPORT float region_wrap_top( RegionWrap* self );
  /**
   * @internal
   * @ingroup OpenShiva
   * Wrapper to return the right of a region
   */
  OPENSHIVA_EXPORT float region_wrap_right( RegionWrap* self );
  /**
   * @internal
   * @ingroup OpenShiva
   * Wrapper to return the bottom of a region
   */
  OPENSHIVA_EXPORT float region_wrap_bottom( RegionWrap* self );
  
  /**
   * @internal
   * @ingroup OpenShiva
   * Wrapper to compute the intersection of region
   */
  OPENSHIVA_EXPORT void region_wrap_intersect( RegionWrap* self, const RegionWrap* reg );
  /**
   * @internal
   * @ingroup OpenShiva
   * Wrapper to compute the union of region
   */
  OPENSHIVA_EXPORT void region_wrap_union( RegionWrap* self, const RegionWrap* reg );
  
  /**
   * @internal
   * @ingroup OpenShiva
   * Wrapper to outset a region
   */
  OPENSHIVA_EXPORT void region_wrap_outset( RegionWrap* self, float amount );
  /**
   * @internal
   * @ingroup OpenShiva
   * Wrapper to inset a region
   */
  OPENSHIVA_EXPORT void region_wrap_inset( RegionWrap* self, float amount );
}

/**
 * @internal
 * @ingroup OpenShiva
 * Wrap a @ref GTLCore::Region
 */
template<typename _T_>
inline RegionWrap* regionToRegionWrap( const _T_& region )
{
  RegionWrap* rw = gtlAllocate<RegionWrap>( );
  rw->x = region.x();
  rw->y = region.y();
  rw->columns = region.columns();
  rw->rows = region.rows();
  return rw;
}

/**
 * @internal
 * @ingroup OpenShiva
 * Unwrap a @ref GTLCore::Region
 */
inline GTLCore::RegionF regionWrapToRegionF( RegionWrap * rwrap)
{
 return GTLCore::RegionF( rwrap->x, rwrap->y, rwrap->columns, rwrap->rows );
}

/**
 * @internal
 * @ingroup OpenShiva
 * Wrap a list @ref GTLCore::Region to an array
 */
inline ArrayWrap* regionListToArrayWrap( const std::list<GTLCore::RegionI> & regions)
{
  ArrayWrap* array = gtlAllocateArray<RegionWrap*>( regions.size() );
  RegionWrap** wrapData = reinterpret_cast<RegionWrap**>(array->data);
  for( std::list<GTLCore::RegionI>::const_iterator it = regions.begin(); it != regions.end(); ++it, ++wrapData)
  {
    *wrapData = regionToRegionWrap( *it );
  }
  return array;
}

#endif
